import Ably, {Types, Realtime} from 'ably';
import ChaturbateAuth, { IPushServiceAuth } from './ChaturbateAuth';
import { IAppContext } from './appContext';

export interface IAblyRealtimeOptions {
  chaturbateAuth: ChaturbateAuth;
  realtimeHost?: string;
  fallbackHosts?: string[];
}

export default class AblyWrapper {
  private logger: any;
  private chaturbateAuth: any;
  private pushServiceAuth?: any;
  public tokenRequest?: any;
  public realtimeClient?: Ably.Realtime;
  private appContext: IAppContext;

  constructor(appContext: IAppContext, opts: IAblyRealtimeOptions) {
    this.logger = appContext.logger;
    this.chaturbateAuth = opts.chaturbateAuth;
    this.pushServiceAuth;
    this.tokenRequest;
    this.appContext = appContext;
    this.realtimeClient;
  }


  getRealtimeUrls(): { restHost: string, realtimeHost: string, fallbackHosts: string[] } {
    const statement = this.appContext.db.prepare(
      `SELECT * FROM chaturbate_auth;`
    );
    const row = statement.get()
    console.log(row)
    return {
      restHost: row.rest_host,
      realtimeHost: row.realtime_host,
      fallbackHosts: [
        row.fallback_hosts_0,
        row.fallback_hosts_1,
        row.fallback_hosts_2,
        row.fallback_hosts_3,
        row.fallback_hosts_4
      ]
    }
  }

  saveRealtimeUrls(
    restHost: string, 
    realtimeHost: string, 
    fallbackHosts: string[]
  ): void {
    const statement = this.appContext.db.prepare(`
      UPDATE chaturbate_auth 
      SET
        rest_host = $restHost,
        realtime_host = $realtimeHost,
        fallback_hosts_0 = $fallbackHosts0,
        fallback_hosts_1 = $fallbackHosts1,
        fallback_hosts_2 = $fallbackHosts2,
        fallback_hosts_3 = $fallbackHosts3,
        fallback_hosts_4 = $fallbackHosts4;
    `)
    statement.run({
      restHost: restHost,
      realtimeHost: realtimeHost,
      fallbackHosts0: fallbackHosts[0],
      fallbackHosts1: fallbackHosts[1],
      fallbackHosts2: fallbackHosts[2],
      fallbackHosts3: fallbackHosts[3],
      fallbackHosts4: fallbackHosts[4],
    })
  }
  

  async getRealtimeClient(
    roomUrl: string,
    permissionsForm: FormData,
    cookie: string
  ): Promise<Realtime> {
    let realtimeUrls = this.getRealtimeUrls()


    if (!realtimeUrls.realtimeHost) {
      throw new Error('realtime host url not found in db');
    }

    if (!realtimeUrls.fallbackHosts.at(0)) {
      throw new Error('realtime fallback urls not found in db')
    }

    let clientOptions: Types.ClientOptions = {
      autoConnect: false,
      closeOnUnload: true,
      transportParams: {
        remainPresentFor: '0'
      },
      restHost: realtimeUrls.realtimeHost,
      realtimeHost: realtimeUrls.realtimeHost,
      fallbackHosts: realtimeUrls.fallbackHosts,
      authCallback: (async (tokenParams, callback) => {
        this.logger.log({
          level: 'debug',
          message: `Ably authCallback. Getting a fresh new Ably TokenRequest.`
        });

        // This is where we get a signed Ably TokenRequest from CB!
        let pushServiceAuth: IPushServiceAuth = await this.chaturbateAuth.getPushServiceAuth(roomUrl, permissionsForm, cookie);

        // cache the realtime host/fallback urls to teh db for later use
        let restHost = pushServiceAuth.settings.rest_host
        let realtimeHost = pushServiceAuth.settings.realtime_host
        let fallbackHosts = pushServiceAuth.settings.fallback_hosts
        console.log(`restHost:${restHost} >> realtimeHost:${realtimeHost} >> fallbackHosts:${fallbackHosts}`)
        await this.saveRealtimeUrls(restHost, realtimeHost, fallbackHosts)

        this.tokenRequest = pushServiceAuth.token_request;

        this.logger.log({
          level: 'debug',
          message: `the following is pushServiceAuth`
        });
        this.logger.log({
          level: 'debug',
          message: JSON.stringify(pushServiceAuth, null, 2)
        });
        this.logger.log({
          level: 'debug',
          message: `Got a new TokenRequest`
        });
        this.logger.log({
          level: 'debug',
          message: JSON.stringify(this.tokenRequest)
        });
        if (!this?.tokenRequest) throw new Error('tokenRequest was empty')

        callback(null, this.tokenRequest);
      })
    }

    this.realtimeClient = new Ably.Realtime.Promise(clientOptions);

    return this.realtimeClient;
  }
}
