import cheerio from 'cheerio'
import { execa } from 'execa'
import pRetry from 'p-retry'

export function getRandomRoomFromCbHtml (body) {
  const $ = cheerio.load(body);
  let roomsRaw = $('a[data-room]');
  let rooms = [];
  $(roomsRaw).each((_, e) => {
    rooms.push($(e).attr('href'));
  })

  // greets https://stackoverflow.com/a/4435017/1004931
  var randomIndex = Math.floor(Math.random() * rooms.length);
  return rooms[randomIndex].replaceAll('/', '');
}


// helper for testing
export const getRandomPlaylist = async (appContext) => {
  const roomName = await getRandomRoom();
  return getPlaylistUrl(appContext, roomName)
}



export const getPlaylistUrl = async (appContext, roomName) => {
  const operation = async () => {
    const { stdout: playlistUrl } = await execa('yt-dlp', ['-g', `https://chaturbate.com/${roomName}`]);
    return playlistUrl.trim();
  };
  return await pRetry(operation, { 
    retries: 6, 
    minTimeout: 2000,
    onFailedAttempt(e) {
      appContext.logger.log({ level: 'debug', message: `error getting playlist url during attempt ${e.attemptNumber}. ${e.retriesLeft} retries remaining.` })
    }
  });
};