// import { createVod } from './strapi.js'
import Stream from './Stream.js'
import { signalStart, sendSignal } from './faye.js'
import { IAppContext } from './appContext.js'
import * as Ably from 'ably/promises.js';
import { application } from 'express';

export const onCbPassword = async (appContext, roomName, message) => {
  appContext.logger.log({ level: 'debug', message: `[ ] room is passworded`})
  // const stmt = appContext.db.prepare(`INSERT INTO passwords VALUES `)
  // @todo log passworded room events
}

export const onCbTip = async (appContext, roomName, message) => {
  appContext.logger.log({ level: 'debug', message: `💲💲💲 [TIP] ${JSON.stringify(message)}` })
  const stmt = appContext.db.prepare(`INSERT INTO tips VALUES ($_room, $name, $encoding, $data_tid, $data_ts, $data_amount, $data_message, $data_history, $data_is_anonymous_tip, $data_to_username, $data_from_username, $data_gender, $data_is_broadcaster, $data_in_fanclub, $data_is_following, $data_is_mod, $data_has_tokens, $data_tipped_recently, $data_tipped_alot_recently, $data_tipped_tons_recently, $data_method, $data_pub_ts)`)
  stmt.run({
    '_room': roomName,
    'name': message?.name || null,
    'encoding': message?.encoding || null,
    'data_tid': message?.data?.tid || null,
    'data_ts': message?.data?.ts || null,
    'data_amount': message?.data?.amount || null,
    'data_message': message?.data?.message || null,
    'data_history': (!!message?.data?.history) ? 1 : 0,
    'data_is_anonymous_tip': (!!message?.data?.is_anonymous_tip) ? 1 : 0,
    'data_to_username': message?.data?.to_username || null,
    'data_from_username': message?.data?.from_username || null,
    'data_gender': message?.data?.gender || null,
    'data_is_broadcaster': (!!message?.data?.is_broadcaster) ? 1 : 0,
    'data_in_fanclub': (!!message?.data?.in_fanclub) ? 1 : 0,
    'data_is_following': (!!message?.data?.is_following) ? 1 : 0,
    'data_is_mod': (!!message?.data?.is_mod) ? 1 : 0,
    'data_has_tokens': (!!message?.data?.has_tokens) ? 1 : 0,
    'data_tipped_recently': (!!message?.data?.tipped_recently) ? 1 : 0,
    'data_tipped_alot_recently': (!!message?.data?.tipped_alot_recently) ? 1 : 0,
    'data_tipped_tons_recently': (!!message?.data?.tipped_tons_recently) ? 1 : 0,
    'data_method': message?.data?.method || null,
    'data_pub_ts': message?.data?.pub_ts || null,
  })
}



export const onCbStart = async (appContext, roomName, message) => {
  appContext.logger.log({ level: 'debug', message: `🟢🟢🟢 [STREAM START] ${JSON.stringify(message)}` })
  const stmt = appContext.db.prepare(`INSERT INTO lifecycles VALUES ($_room, $name, $id, $encoding, $data_tid, $data_ts, $data_status, $data_message, $data_hash, $data_method, $data_pub_ts)`)
  stmt.run({
    '_room': roomName,
    'name': message?.name || null,
    'id': message?.id || null,
    'encoding': message?.encoding || null,
    'data_tid': message?.data?.tid || null,
    'data_ts': message?.data?.status || null,
    'data_status': message?.data?.status || null,
    'data_message': message?.data?.message || null,
    'data_hash': message?.data?.hash || null,
    'data_method': message?.data?.method || null,
    'data_pub_ts': message?.data?.pub_ts || null,
  })
  appContext.sound.alert()

  if (appContext.cb.getPlaylistUrl === undefined) {
    console.log('getPlaylistUrl is undefined')
    console.log(appContext)
    throw new Error('getPlaylistUrl missing!');
  } else {
    console.log(`appContext.cb.getPlaylistUrl is defined and it is ${appContext.cb.getPlaylistUrl}`)
  }


  const playlistUrl = await appContext.cb.getPlaylistUrl(appContext, roomName)

  try {
    signalStart(appContext, roomName, playlistUrl)
  } catch (e) {
    console.error(`There was a problem while attempting to signalStart via faye`)
    console.error(e)
  }


  // clear any offline timer which might be running. 
  // this is to prevent temporary outages from being counted as a new stream
  // @todo this is faulty. more thought needed to implement a timer.
  // !!appContext.roomTimers[roomName].offline && clearTimeout(appContext.roomTimers[roomName].offline)
}

export const onCbNotice = (appContext: IAppContext, roomName: string, message: Ably.Types.Message) => {
  appContext.logger.log({ level: 'debug', message: `📢📢📢 [NOTICE] roomName=${roomName} ${JSON.stringify(message)}` })
  const stmt = appContext.db.prepare(`INSERT INTO notices VALUES (
      $id,
      $_room,
      $data_messages,
      $data_tid,
      $data_ts
    )`);
    stmt.run({
      'id': message?.id || null,
      '_room': roomName,
      'data_messages': message?.data?.messages.join('\n') || null,
      'data_tid': message?.data?.tid || null,
      'data_ts': message?.data?.ts || null,
    });
}

export const onCbStop = (appContext, roomName, message) => {
  appContext.logger.log({ level: 'debug', message: `🛑🛑🛑 [STREAM STOP] ${JSON.stringify(message)}` })
  const stmt = appContext.db.prepare(`INSERT INTO lifecycles VALUES (
    $_room,
    $name,
    $id,
    $encoding,
    $data_tid,
    $data_ts,
    $data_status,
    $data_message,
    $data_hash,
    $data_method,
    $data_pub_ts
  )`)
  stmt.run({
    '_room': roomName,
    'name': message?.name || null,
    'id': message?.id || null,
    'encoding': message?.encoding || null,
    'data_tid': message?.data?.tid || null,
    'data_ts': message?.data?.ts || null,
    'data_status': message?.data?.status || null,
    'data_message': message?.data?.message || null,
    'data_hash': message?.data?.hash || null,
    'data_method': message?.data?.method || null,
    'data_pub_ts': message?.data?.pub_ts || null,
  })


  sendSignal(appContext, { signal: 'stop', room: roomName })



  // start offline timer which runs if 5 minutes without a start event elapses
  // @todo this is FAULTY
  // @see https://github.com/insanity54/futureporn/issues/206
  // appContext.roomTimers[roomName].offline = setTimeout(() => createStreamVOD(appContext, message), 1000*60*5) 
}


/**
 * 
 * onCbStatus
 * 
 * 
 * CB changed their systems sometime around fall 2023 to where mutiple event types are grouped into a single channel. Due to this, a status callback might actually contain a chat message, so we need to test _topic and see what type of event data we are dealing with.
 * @see https://gitea.futureporn.net/futureporn/futureporn-scout/issues/3
 * 
 * @param appContext 
 * @param roomName 
 * @param message 
 * 
 * @deprecated I don't think CB uses RoomStatusTopic anymore.
 */
export const onCbStatus = async (appContext: IAppContext, roomName: string, message: Ably.Types.Message) => {
  if (message.data._topic === 'RoomMessageTopic') {
    onCbMessage(appContext, roomName, message)
  } else if (message.data?.status === 'public') {
    onCbStart(appContext, roomName, message)
  } else if (message.data?.status === 'offline') {
    onCbStop(appContext, roomName, message)
  } else {
    appContext.logger.log({ level: 'warn', message: `⚠️⚠️⚠️ there was an incomming message which we dont know how to handle. `})
    appContext.logger.log({ level: 'warn', message: message })
  }
}

/**
 * 
 * @todo
 * @param appContext 
 * @param message 
 */
export const createStreamVOD = (appContext: IAppContext, message: Ably.Types.Message) => {
  // find the relevant StreamSegments
  // group the StreamSegments into a Stream
  // create a vod using the start/stop timestamps of the Stream, and the messages falling within those timestamps

  const stream = new Stream()
  stream.getStreamSegmentsUsingEndTs(message.data.ts)
  stream.getChat()
  // stream.uploadToStrapi(appContext)
}

export const onCbTitle = (appContext, roomName, message) => {
  appContext.logger.log({ level: 'debug', message: `title changed! ${JSON.stringify(message)}`})
  const stmt = appContext.db.prepare(`INSERT INTO titles VALUES (
    $_room,
    $name,
    $id,
    $encoding,
    $data_tid,
    $data_ts,
    $data_title,
    $data_method,
    $data_pub_ts
  )`)
  stmt.run({
    '_room': roomName,
    'name': message?.name || null,
    'id': message?.id || null,
    'encoding': message?.encoding || null,
    'data_tid': message?.data?.tid || null,
    'data_ts': message?.data?.ts || null,
    'data_title': message?.data?.title || null,
    'data_method': message?.data?.method || null,
    'data_pub_ts': message?.data?.pub_ts || null,
  })
}

export const onCbSilence = (appContext, roomName, message) => {
  appContext.logger.log({ level: 'debug', message: `Bad boys get busted! ${JSON.stringify(message)}`})
  const stmt = appContext.db.prepare(`INSERT INTO silences VALUES (
    $_room,
    $name,
    $id,
    $encoding,
    $data_tid,
    $data_ts,
    $data_username,
    $data_from_username,
    $data_method,
    $data_pub_ts
  )`)
  stmt.run({
    '_room': roomName,
    'name': message?.name || null,
    'id': message?.id || null,
    'encoding': message?.encoding || null,
    'data_tid': message?.data?.tid || null,
    'data_ts': message?.data?.ts || null,
    'data_username': message?.data?.username || null,
    'data_from_username': message?.data?.from_username || null,
    'data_method': message?.data?.method || null,
    'data_pub_ts': message?.data?.pub_ts || null,
  })
}


export const onCbMessage = (appContext, roomName, message) => {
  appContext.logger.log({ level: 'debug', message: `💬💬💬 [CHAT MESSAGE] ${JSON.stringify(message)}` })

  const stmt = appContext.db.prepare(`INSERT INTO messages VALUES (
    $_room,
    $name, 
    $id, 
    $encoding, 
    $data_tid, 
    $data_ts, 
    $data_message, 
    $data_font_family,
    $data_font_color, 
    $data_id, 
    $data_background, 
    $data_from_user_username,
    $data_from_user_gender,
    $data_from_user_is_broadcaster,
    $data_from_user_in_fanclub,
    $data_from_user_is_following,
    $data_from_user_is_mod,
    $data_from_user_has_tokens,
    $data_from_user_tipped_recently,
    $data_from_user_tipped_alot_recently,
    $data_from_user_tipped_tons_recently,
    $data_method,
    $data_pub_ts
  )`);
  stmt.run({
    '_room': roomName,
    'name': message?.name || null,
    'id': message?.id || null,
    'encoding': message?.encoding || null,
    'data_tid': message?.data?.tid || null,
    'data_ts': message?.data?.ts || null,
    'data_message': message?.data?.message || null,
    'data_font_family': message?.data?.font_family || null,
    'data_font_color': message?.data?.font_color || null,
    'data_id': message?.data?.id || null,
    'data_background': message?.data?.background || null,
    'data_from_user_username': message?.data?.from_user?.username || null,
    'data_from_user_gender': message?.data?.from_user?.gender || null,
    'data_from_user_is_broadcaster': (!!message?.data?.from_user?.is_broadcaster) ? 1 : 0,
    'data_from_user_in_fanclub': (!!message?.data?.from_user?.in_fanclub) ? 1 : 0,
    'data_from_user_is_following': (!!message?.data?.from_user?.is_following) ? 1 : 0,
    'data_from_user_is_mod': (!!message?.data?.from_user?.is_mod) ? 1 : 0,
    'data_from_user_has_tokens': (!!message?.data?.from_user?.has_tokens) ? 1 : 0,
    'data_from_user_tipped_recently': (!!message?.data?.from_user?.tipped_recently) ? 1 : 0,
    'data_from_user_tipped_alot_recently': (!!message?.data?.from_user?.tipped_alot_recently) ? 1 : 0,
    'data_from_user_tipped_tons_recently': (!!message?.data?.from_user?.tipped_tons_recently) ? 1 : 0,
    'data_method': message?.method || null,
    'data_pub_ts': message?.pub_ts || null,
  });

}
