import winston, { Logger, LoggerOptions } from 'winston';

export const loggerFactory = (options: LoggerOptions): Logger => {
  const mergedOptions: LoggerOptions = {
    level: 'info',
    defaultMeta: { service: 'futureporn' },
    format: winston.format.timestamp(),
    ...options
  };

  const logger: Logger = winston.createLogger(mergedOptions);

  if (process.env.NODE_ENV !== 'production') {
    logger.add(new winston.transports.Console({
      level: 'debug',
      format: winston.format.simple()
    }));
  } else {
    logger.add(new winston.transports.Console({
      level: 'info',
      format: winston.format.json()
    }));
  }

  return logger;
};
