
import * as Ably from 'ably/promises.js';
import { IPushServiceAuth, getPushServiceAuth } from './headless.js';
import { iPushServiceAuthSchema } from './headlessSchema.js';

export async function getSuperRealtimeClient(room: string, psa: IPushServiceAuth): Promise<Ably.Realtime> {
    if ((/https?:\/\//).test(room)) throw new Error(`getSuperRealtimeClient received a URL but thats not what it needs. It needs only the room name, ex: 'projektmelody'`);
    iPushServiceAuthSchema.parse(psa);
    const authCallback: Ably.Types.AuthOptions['authCallback'] = (async (_, callback) => {
        console.log(`Ably authCallback. Getting a fresh new Ably TokenRequest.`);
        const roomUrl = new URL(`https://chaturbate.com/${room}`);
        const tokenRequest = psa.token_request;
        if (!tokenRequest) throw new Error('tokenRequest was empty');
        callback(null, tokenRequest);
    });
    const options: Ably.Types.ClientOptions = {
        autoConnect: false,
        closeOnUnload: true,
        transportParams: {
            remainPresentFor: '0'
        },
        restHost: psa.settings.rest_host,
        realtimeHost: psa.settings.realtime_host,
        fallbackHosts: psa.settings.fallback_hosts,
        authCallback: authCallback
    };
    const realtime = new Ably.Realtime(options);
    return realtime;
    
}