import faye from 'faye'
import nunjucks from 'nunjucks'
import Fastify from 'fastify'
import { IAppContext } from './appContext'

export default function webserver (appContext: IAppContext): void {

    // var server = http.createServer(),
    //     bayeux = new faye.NodeAdapter({mount: '/'});

    // bayeux.attach(server);
    // server.listen(appContext.env?.PORT || 6934);
    const { SCHEME, HOST, PORT } = appContext.env
    const fastify = Fastify({
        logger: true
    })


    // Configure Nunjucks template engine
    nunjucks.configure('.', {
        autoescape: true,
        express: fastify,
    });
    
    // Faye server
    const bayeux = new faye.NodeAdapter({ mount: '/faye', timeout: 45 });
    
    // Faye client script
    const fayeClientScript = `
        <script src="/faye/client.js"></script>
        <script>
        const client = new Faye.Client('${SCHEME}://${HOST}:${PORT}/faye');
        // Faye client initialization code
        // ...
        </script>
    `;
    
    // Fastify route for rendering the index.html template
    fastify.get('/', (req, reply) => {
        const data = {
            SCHEME,
            PORT,
            HOST,
        };
        
        reply.header('Content-Type', 'text/html');
        reply.send(nunjucks.render('index.njk', { data, fayeClientScript }));
    });
    
    // Faye server listening
    bayeux.attach(fastify.server);
    
    // Start the server
    fastify.listen(PORT, HOST, (err, address) => {
        if (err) {
        console.error(err);
        process.exit(1);
        }
        console.log(`Server listening on ${address}`);
    });

    
}