import { expect } from 'chai';
import path from 'node:path';
import ChaturbateAuth from '../../src/ChaturbateAuth.js';
import gotClient from '../../src/gotClient.js'
import fs from 'node:fs';
import { loggerFactory } from '../../src/logger.js';

describe('ChaturbateAuth Integration Tests', () => {

  const randomDirectoryName = `test-${Math.random().toString(36).substring(2)}`;
  const tmpDirectory = path.join('/tmp', randomDirectoryName);

  // Create the random directory in /tmp
  fs.mkdirSync(tmpDirectory);

  let appContext = {
    logger: loggerFactory(),
    dataDir: tmpDirectory,
    gotClient
  };

  let chaturbateAuth;

  beforeEach(() => {
    chaturbateAuth = new ChaturbateAuth(appContext);
  });

  describe('Compatibility', function () {
    xit('should use known channel names', function () {

      const channels = {
        'GlobalPushServiceBackendChangeTopic#GlobalPushServiceBackendChangeTopic': 'global:push_service',
        'RoomAnonPresenceTopic#RoomAnonPresenceTopic:G0TWFS5': 'room_anon:presence:G0TWFS5:16',
        'QualityUpdateTopic#QualityUpdateTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomMessageTopic#RoomMessageTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomFanClubJoinedTopic#RoomFanClubJoinedTopic:G0TWFS5': 'room:fanclub:G0TWFS5',
        'RoomPurchaseTopic#RoomPurchaseTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomNoticeTopic#RoomNoticeTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomTipAlertTopic#RoomTipAlertTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomShortcodeTopic#RoomShortcodeTopic:G0TWFS5': 'room:shortcode:G0TWFS5',
        'RoomPasswordProtectedTopic#RoomPasswordProtectedTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomModeratorPromotedTopic#RoomModeratorPromotedTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomModeratorRevokedTopic#RoomModeratorRevokedTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomStatusTopic#RoomStatusTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomTitleChangeTopic#RoomTitleChangeTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomSilenceTopic#RoomSilenceTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomKickTopic#RoomKickTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomUpdateTopic#RoomUpdateTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomSettingsTopic#RoomSettingsTopic:G0TWFS5': 'room:grouped:G0TWFS5:16',
        'RoomEnterLeaveTopic#RoomEnterLeaveTopic:G0TWFS5': 'room:enter_leave:G0TWFS5',
        'GameUpdateTopic#GameUpdateTopic:G0TWFS5': 'room:grouped:G0TWFS5:16'
      }
    })
  })
  // Add more test cases for other methods as needed

  // You can also add tests for error cases, edge cases, etc.
});

