import puppeteer from 'puppeteer';
import { expect } from 'chai';
import gotClient from '../../src/gotClient.js';
import { getRandomRoom, getAblyAgentVersionSemver, getSemverNumber, getPushServiceAuth } from '../../src/headless.js';
import { readFile } from 'fs/promises';
import path from 'path';
import { dirname } from 'path';
import { fileURLToPath } from 'url';

const __dirname = dirname(fileURLToPath(import.meta.url));

const getLocalAblyVersion = async function () {
    const content = await readFile(path.join(__dirname, '../../package.json'), { encoding: 'utf-8' });
    const json = JSON.parse(content, null, 2);
    if (!json?.dependencies?.ably) throw new Error('failed to get ably version');
    return json.dependencies.ably;
}

// Once request interception is enabled,
// every request will stall unless it's continued, responded or aborted.

describe('headless browser', function () {
    // xdescribe('sanity checks', async function (done) {
    //     const browser = await puppeteer.launch({ headless: false });
    //     const page = await browser.newPage();
    //     await page.setRequestInterception(true);
    //     page.on('request', interceptedRequest => {
    //         if (interceptedRequest.isInterceptResolutionHandled()) return;
    //         if (
    //             interceptedRequest.url().endsWith('.png') ||
    //             interceptedRequest.url().endsWith('.jpg') ||
    //             interceptedRequest.url().endsWith('.webp') ||
    //             interceptedRequest.url().endsWith('.avif') ||
    //             interceptedRequest.url().endsWith('.svg')
    //         )
    //             interceptedRequest.abort();
    //         else interceptedRequest.continue();
    //     });
    //     await page.goto('https://google.com');
    //     await browser.close();
    //     done();
    // })
    describe('helpers', function () {
        describe('getLocalAblyVersion', function () {
            it('should return a version string', async function () {
                const ver = await getLocalAblyVersion();
                expect(ver).to.be.a('string');
                expect(ver).to.match(/^(0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(?:-((?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(?:\.(?:0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?(?:\+([0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$/);
                return ver;
            })
        })
        describe('getSemverNumber', function () {
            it('should accept an agent string and return a semver number', function () {
                const input = 'ably-js/1.2.37 browser'
                const output = '1.2.37'
                expect(getSemverNumber(input)).to.equal(output)
            })
        })
    })

    describe('chaturbate', function () {
        this.timeout(1000*30);
        it('should get a token_request', async function () {
            const room = await getRandomRoom();
            const roomUrl = `https://chaturbate.com/${room}`;
            const psa = await getPushServiceAuth(roomUrl);
            expect(psa).to.be.a('object');
            expect(psa).to.have.a.property('token');
            expect(psa).to.have.a.property('channels');
            expect(psa).to.have.a.property('failures');
            expect(psa).to.have.a.property('token_request');
            expect(psa).to.have.a.property('client_id');
            expect(psa).to.have.a.property('settings');
            expect(psa.token).to.be.a('string').that.is.lengthOf.above(2);
            expect(psa.channels).to.be.a('object');
            expect(psa.failures).to.be.a('object').that.is.empty;
            expect(psa.settings).to.have.a.property('rest_host');
            expect(psa.settings).to.have.a.property('realtime_host');
        });

        it('cb ably version should match local ably agent version', async function () {
            const cbAblyVersion = await getAblyAgentVersionSemver();
            const localAblyVersion = await getLocalAblyVersion();
            expect(cbAblyVersion).to.equal(localAblyVersion);
        });

    })

})