
import {expect} from "chai";

import { sendSignal, signalStart } from '../../src/faye.ts'
import { getRandomPlaylist } from '../../src/cb.js'
import Faye from 'faye'
import dotenv from 'dotenv'
dotenv.config()


// for reference of _state contants,
// @see https://github.com/faye/faye/blob/60141e8d3942a88ba3de6a9b3aef9503bc1bb1e6/src/protocol/client.js#L178
const UNCONNECTED = 1
const CONNECTING = 2
const CONNECTED = 3
const DISCONNECTED = 4

describe('pubsub', function () {

  if (!process.env.PUBSUB_SERVER_URL) throw new Error('PUBSUB_SERVER_URL missing in env');

  const fakeAppContext = {
    faye: new Faye.Client(process.env.PUBSUB_SERVER_URL),
    logger: { log: (txt) => console.log(JSON.stringify(txt)) }
  }


  // describe('connection', function () {
  //   it('should be connected', function (done) {
  //     conn.callback(function () {
        
  //       done()
  //     })
  //   })
  // })

  describe('sendSignal', function () {
    it('should send', async function () {
      console.log(`sending pubsub signal to ${process.env.PUBSUB_SERVER_URL}`)
      await sendSignal(fakeAppContext, 'IT WORKS!')
      expect(fakeAppContext.faye._state).to.equal(CONNECTED)
    })
  })

  describe('signalStart', function () {
    it('should send a signal to futureporn-capture via futureporn-realtime', async function () {
      this.timeout(1000*15)
      const playlistUrl = 'https://example.com/my-stream.m3u8'
      await signalStart(fakeAppContext, 'projektmelody', playlistUrl)
      console.log('signal done')
    });
  })

  this.afterAll(function () {
    console.log('disconnecting...')
    fakeAppContext.faye.disconnect()
  })

});
