import Room from '../../src/Room.js';
import ChaturbateAuth from '../../src/ChaturbateAuth.js';
import * as Ably from 'ably/promises';
import chai, { expect } from "chai";
import chaiEvents from 'chai-events';
import gotClient from '../../src/gotClient.js';
import path from 'node:path';
import os from 'node:os';
import { getPushServiceAuth, getRandomRoom } from '../../src/headless.js';

chai.use(chaiEvents);


describe('realtime', function () {

    describe(`Ably`, function () {
        this.timeout(60 * 1000);
        it(`should subscribe to a room's events`, async function () {
            const room = await getRandomRoom();
            const authCallback: Ably.Types.AuthOptions['authCallback'] = (async (_, callback) => {
                console.log(`Ably authCallback. Getting a fresh new Ably TokenRequest.`);

                // This is where we get a signed Ably TokenRequest from CB!
                const roomUrl = `https://chaturbate.com/${room}`;
                let pushServiceAuth = await getPushServiceAuth(roomUrl);
                console.log(pushServiceAuth);

                const tokenRequest = pushServiceAuth.token_request;

                console.log(`Got a new TokenRequest`);
                console.log(JSON.stringify(tokenRequest, null, 2));
                if (!tokenRequest) throw new Error('tokenRequest was empty');

                callback(null, tokenRequest);
            })
            const options: Ably.Types.ClientOptions = {
                autoConnect: false,
                closeOnUnload: true,
                transportParams: {
                    remainPresentFor: '0'
                },
                restHost: 'realtime.pa.highwebmedia.com',
                realtimeHost: 'realtime.pa.highwebmedia.com',
                fallbackHosts: [
                    'a-fallback.pa.highwebmedia.com',
                    'b-fallback.pa.highwebmedia.com',
                    'c-fallback.pa.highwebmedia.com',
                    'd-fallback.pa.highwebmedia.com',
                    'e-fallback.pa.highwebmedia.com'
                ],
                authCallback: authCallback
            };
            const realtime = new Ably.Realtime(options);
            const goodEnd = new Promise<void>((resolve) => {
                realtime.connection.once('connected', (idk) => {
                    console.log(`  [*] connected! ${JSON.stringify(idk, null, 2)}`);
                    realtime.close();
                    resolve()
                });
                console.log('>>> connecting to realtime');
                realtime.connect();
            });
            const badEnd = new Promise((_, reject) => {
                setTimeout(() => {
                    reject('timeout waiting for realtime connection');
                }, 30 * 1000);
            });
            return Promise.race([goodEnd, badEnd]);
        })
    })

    // xdescribe('Can I use my own permission form?', function () {
    //     this.timeout(60*1000)
    //     it('should subscribe to Ably realtime, error free', async function () {


    //         const fakeAppContext = {
    //             logger: {
    //                 log: (msg) => console.log(JSON.stringify(msg)),
    //             },
    //             dataDir: path.join(os.homedir(), '.local/share/futureporn-scout'),
    //             gotClient
    //         }

    //         expect(fakeAppContext.logger.log).to.be.a('function')

    //         const cb = new ChaturbateAuth(fakeAppContext)
    //         const ably = new Realtime(fakeAppContext, { chaturbate: cb })
    //         const rooms = [
    //             'projektmelody',
    //             'el_xox',
    //             'skyeanette',
    //             // 'athena_airis'
    //         ].map((r) => new Room(fakeAppContext, { name: r, cb, ably }))

    //         const elRoom = rooms[1]
    //         expect(elRoom.url).to.equal('https://chaturbate.com/el_xox/')
    //         expect(elRoom.id).to.be.a('promise')

    //         // get cookie from cb. We only need one for `chaturbate.com/` (not one per room.)
    //         await cb.fetchCookiesIfNeeded(elRoom.url)
    //         const csrfToken = await cb.getCsrfToken(elRoom.url) // doesn't really matter which roomUrl for this token, as long as it is on chaturbate.com domain
    //         const cookieString = await cb.cookieJar.getCookieString(elRoom.url)

    //         console.log('here is the cookiestring')
    //         console.log(cookieString)

    //         await Promise.allSettled((rooms.map((r) => r.id)))

    //         expect(elRoom.id).to.equal('JQ2YJS5')

    //         // @todo in index.js, flag any rooms which don't have an ID. This would suggest a passworded room.

    //         const openRooms = rooms.filter((r) => (!!r.id))

    //         // get permission form for all the rooms we are interested in
    //         const permissionForm = Room.getPermissionsForm(openRooms.map((r) => r.id), ['status', 'silence'], csrfToken)

    //         for (let pair of permissionForm.entries()) {
    //             console.log(pair[0] + ': ' + pair[1]);
    //         }

    //         // get signed ably tokenrequest from cb
    //         const token = await cb.getTokenRequest(elRoom.url, permissionForm, cookieString)

    //         expect(token).to.have.property('keyName')
    //         console.log(token)

    //         // get realtime connection with merged permission form(?)







    //     })
    // })

})
